@@ -9,8 +9,6 @@
 
 """
 
-import os
-import json
 import re
 
 from docutils import nodes
@@ -24,7 +22,7 @@ from pygments.util import ClassNotFound
 from sphinx import addnodes
 from sphinx.roles import XRefRole
 from sphinx.domains import Domain, ObjType, Index
-from sphinx.directives import ObjectDescription
+from sphinx.directives import ObjectDescription, directives
 from sphinx.util.nodes import make_refnode
 from sphinx.util.docfields import GroupedField, TypedField
 
@@ -214,7 +212,7 @@ def sort_by_method(entries):
 
 
 def http_resource_anchor(method, path):
-    path = re.sub(r'[<>:/]', '-', path)
+    path = re.sub(r'[{}]', '', re.sub(r'[<>:/]', '-', path))
     return method.lower() + '-' + path
 
 
@@ -260,6 +258,12 @@ class HTTPResource(ObjectDescription):
                      names=('statuscode', 'status', 'code'))
     ]
 
+    option_spec = {
+        'deprecated': directives.flag,
+        'noindex': directives.flag,
+        'synopsis': lambda x: x,
+    }
+
     method = NotImplemented
 
     def handle_signature(self, sig, signode):
@@ -295,7 +299,11 @@ class HTTPResource(ObjectDescription):
 
     def add_target_and_index(self, name_cls, sig, signode):
         signode['ids'].append(http_resource_anchor(*name_cls[1:]))
-        self.env.domaindata['http'][self.method][sig] = (self.env.docname, '')
+        if 'noindex' not in self.options:
+            self.env.domaindata['http'][self.method][sig] = (
+                self.env.docname,
+                self.options.get('synopsis', ''),
+                'deprecated' in self.options)
 
     def get_index_text(self, modname, name):
         return ''
@@ -454,8 +462,6 @@ class HTTPIndex(Index):
     name = 'api'
     localname = 'HTTP API Reference'
     shortname = 'API Reference'
-    api_descr = json.load(open(os.path.join(os.path.dirname(__file__),
-                                            'http-api-descr.json')))
 
     def generate(self, docnames=None):
         content = {}
@@ -469,7 +475,7 @@ class HTTPIndex(Index):
             entries.append([
                 entry_name, 0, info[0],
                 http_resource_anchor(method, path),
-                '', '', self.api_descr.get(entry_name, '')
+                '', 'Deprecated' if info[2] else '', info[1]
             ])
         items = sorted(
             (path, sort_by_method(entries))
