@@ -133,6 +133,7 @@ var Mime = (function() {
 ////
 
 var Render = (function() {
+  var new_header = false;
   var chunks = [];
   
   
@@ -140,6 +141,7 @@ var Render = (function() {
   var startResp = {};
   function start(resp) {
     startResp = resp || {};
+    new_header = true;
   };
 
   function sendStart() {
@@ -147,6 +149,7 @@ var Render = (function() {
     respond(["start", chunks, startResp]);
     chunks = [];
     startResp = {};
+    new_header = false;
   }
 
   function applyContentType(resp, responseContentType) {
@@ -162,7 +165,13 @@ var Render = (function() {
   };
 
   function blowChunks(label) {
-    respond([label||"chunks", chunks]);
+    if (new_header) {
+      respond([label||"chunks", chunks, startResp]);
+      new_header = false;
+    }
+    else {
+      respond([label||"chunks", chunks]);
+    }
     chunks = [];
   };
 
@@ -252,7 +261,7 @@ var Render = (function() {
       if (args[0] === null && isDocRequestPath(args[1])) {
         throw(["error", "not_found", "document not found"]);
       } else {
-        renderError(e, fun.toSource());
+        renderError(e, fun.toString());
       }
     }
   };
@@ -272,7 +281,7 @@ var Render = (function() {
         throw(["error", "render_error", "undefined response from update function"]);      
       }
     } catch(e) {
-      renderError(e, fun.toSource());
+      renderError(e, fun.toString());
     }
   };
 
@@ -281,6 +290,7 @@ var Render = (function() {
     lastRow = false;
     chunks = [];
     startResp = {};
+    new_header = false;
   };
 
   function runList(listFun, ddoc, args) {
@@ -300,7 +310,7 @@ var Render = (function() {
       }
       blowChunks("end");
     } catch(e) {
-      renderError(e, listFun.toSource());
+      renderError(e, listFun.toString());
     }
   };
 
@@ -308,7 +318,9 @@ var Render = (function() {
     if (e.error && e.reason || e[0] == "error" || e[0] == "fatal") {
       throw(e);
     } else {
-      var logMessage = "function raised error: "+e.toSource()+" \nstacktrace: "+e.stack;
+      var logMessage = "function raised error: " +
+                       (e.toSource ? e.toSource() : e.toString()) + " \n" +
+                       "stacktrace: " + e.stack;
       log(logMessage);
       throw(["error", "render_error", logMessage]);
     }
