@@ -1,3 +1,15 @@
+// Licensed under the Apache License, Version 2.0 (the "License"); you may not
+// use this file except in compliance with the License. You may obtain a copy of
+// the License at
+//
+//   http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+// License for the specific language governing permissions and limitations under
+// the License.
+
 // Set the require.js configuration for your application.
 require.config({
 
@@ -13,22 +25,22 @@ require.config({
     jquery: "../assets/js/libs/jquery",
     lodash: "../assets/js/libs/lodash",
     backbone: "../assets/js/libs/backbone",
-    "backbone.layoutmanger": "../assets/js/plugins/backbone.layoutmanager",
+    "backbone.layoutmanager": "../assets/js/plugins/backbone.layoutmanager",
     bootstrap: "../assets/js/libs/bootstrap",
-    codemirror: "../assets/js/libs/codemirror",
-    jshint: "../assets/js/libs/jshint",
     spin: "../assets/js/libs/spin.min",
     d3: "../assets/js/libs/d3",
-    "nv.d3": "../assets/js/libs/nv.d3"
+    "nv.d3": "../assets/js/libs/nv.d3",
+    "ace":"../assets/js/libs/ace",
+    "cloudant.pagingcollection": "../assets/js/plugins/cloudant.pagingcollection"
   },
 
   baseUrl: '/',
 
   map: {
     "*": {
-      'underscore': 'lodash'
+      'underscore': 'lodash',
+      'api':'core/api'
     }
-
   },
 
   shim: {
@@ -42,21 +54,11 @@ require.config({
       deps: ["jquery"],
       exports: "Bootstrap"
     },
-
-    codemirror: {
-      deps: ["jquery"],
-      exports: "CodeMirror"
-    },
-
-    jshint: {
-      deps: ["jquery"],
-      exports: "JSHINT"
-    },
-
-    "plugins/codemirror-javascript": ["codemirror"],
-
+    
     "plugins/prettify": [],
+    "plugins/beautify": [],
 
     "plugins/jquery.form": ["jquery"]
   }
 });
+
