@@ -25,7 +25,7 @@ LANGUAGE = ENV["QS_LANG"] || "js"
 
 puts "Running query server specs for #{LANGUAGE} query server"
 
-require 'spec'
+require 'rspec'
 require 'json'
 
 class OSProcessRunner
@@ -821,4 +821,4 @@ end
 describe "thank you for using the tests" do
   it "for more info run with QS_TRACE=true or see query_server_spec.rb file header" do
   end
-end
\ No newline at end of file
+end
